package dev.pfaff.altargui.util.listenable;

import java.util.function.Consumer;

public abstract class AbstractValue<T> extends AbstractSignal implements WritableValue<T> {
	public AbstractValue() {}

	@Override
	public final ListenerDisposer listen(Consumer<T> listener) {
		if (listen((Object)listener)) {
			listener.accept(read());
		}
		return () -> removeListener(listener);
	}

	protected final void notifyUpdateValue(T value) {
		forEachListener(listener -> {
			switch (listener) {
				case Runnable runnable -> runnable.run();
				case Consumer consumer -> consumer.accept(value);
				default -> {}
			}
		});
	}

	/**
	 * @return whether the value was updated.
	 */
	public abstract boolean valueNoNotify(T value);

	public final void write(T value) {
		if (this.valueNoNotify(value)) {
			notifyUpdateValue(value);
		}
	}
}
