package dev.pfaff.altargui.util.listenable;

import java.util.function.Consumer;
import java.util.function.Function;

public interface Value<T> extends Signal {
	/**
	 * This <strong>may</strong> be expensive!
	 */
	T read();

	/**
	 * Adds a listener. The listener <em>may</em> not be added again if already added. When added, it is guaranteed to
	 * be invoked immediately with the current value.
	 */
	ListenerDisposer listen(Consumer<T> listener);

	@Override
	default ListenerDisposer listen(Runnable listener) {
		return listen(_ -> listener.run());
	}

	default <R> Value<R> map(Function<T, R> mapper) {
		return new DerivedValue<>(this, mapper);
	}
}
